@@ -1,6 +1,8 @@
 from .DataPackageServer import FlaskFunctions, Path, dp_directory, os, app, eventlet, const
 from FreeTAKServer.controllers.configuration.LoggingConstants import LoggingConstants
 from FreeTAKServer.controllers.CreateLoggerController import CreateLoggerController
+from FreeTAKServer.controllers.configuration.DataPackageServerConstants import DataPackageServerConstants
+
 loggingConstants = LoggingConstants()
 logger = CreateLoggerController("TCPDataPackageServer").getLogger()
 
@@ -22,7 +24,7 @@ class TCPDataPackageService(FlaskFunctions):
             super().setHTTPPORT(HTTPPORT)
             super().setIP(IP)
             super().setPIPE(PIPE)
-            wsgi.server(eventlet.listen((IP, HTTPPORT)), app)
+            wsgi.server(eventlet.listen((DataPackageServerConstants().IP, HTTPPORT)), app)
 
 
         except Exception as e:
