@@ -1,7 +1,8 @@
+from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
 from FreeTAKServer.model.FTSModelVariables.SubmissionTimeVariables import SubmissionTimeVariables as vars
 import datetime as dt
 
-class SubmissionTime:
+class SubmissionTime(FTSProtocolObject):
     def __init__(self):
         self.INTAG = None
 
